<?php

namespace Drupal\hubspot_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\hubspot_api\Services\OAuth as OAuthService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller Class for OAuth Functionality.
 */
class OAuth extends ControllerBase {

  /**
   * The OAuth service.
   *
   * @var \Drupal\hubspot_api\Services\OAuth
   */
  protected OAuthService $oauthService;

  /**
   * The Request Stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * Constructs an OAuth object.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\hubspot_api\Services\OAuth $oauthService
   *   The OAuth service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   */
  public function __construct(
    MessengerInterface $messenger,
    OAuthService $oauthService,
    RequestStack $requestStack
  ) {
    $this->messenger = $messenger;
    $this->oauthService = $oauthService;
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('hubspot_api.oauth'),
      $container->get('request_stack')
    );
  }

  /**
   * Makes the redirect.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirect response.
   */
  public function oauthRedirect(): RedirectResponse {
    $this->processRequest();

    return $this->redirect('hubspot_api.settings');
  }

  /**
   * Processes the request from Hubspot's OAuth redirect.
   *
   * Grabs the 'code' sent by Hubspot in the url. This code is then passed along
   * to save the tokens created.
   *
   * @return void
   *   Nothing.
   */
  protected function processRequest(): void {
    $currentRequestQuery = $this->requestStack->getCurrentRequest()->query;

    if ($currentRequestQuery->get('error') == 'access_denied') {
      $this->messenger->addError($this->t('Access was denied to HubSpot API.'));
      return;
    }

    $code = $currentRequestQuery->get('code');
    if (empty($code)) {
      $this->messenger->addError($this->t('Missing response from Hubspot API. Check the error logs.'));
      return;
    }

    $tokens = $this->oauthService->getTokensByCode($code);
    if (empty($tokens)) {
      $this->messenger->addError($this->t('Unable to retrieve tokens from Hubspot API. Check the error logs.'));
      return;
    }
    $this->oauthService->saveTokens($tokens);
  }

}
