<?php

namespace Drupal\hubspot_api;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\State\StateInterface;
use Drupal\hubspot_api\Enum\HubSpotEnum;
use Drupal\hubspot_api\Services\OAuth;
use HubSpot\Discovery\Discovery;
use HubSpot\Factory;

/**
 * Hubspot API Manager.
 */
class Manager implements ManagerInterface {

  /**
   * The config for HubSpot.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * Logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * The Hubspot OAuth service.
   *
   * @var \Drupal\hubspot_api\Services\OAuth
   */
  protected OAuth $oauthService;

  /**
   * The state key/value store.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * Constructs a new HubSpot service instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   The logger factory.
   * @param \Drupal\hubspot_api\Services\OAuth $oauthService
   *   The OAuth service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state.
   */
  public function __construct(ConfigFactoryInterface $configFactory, LoggerChannelFactoryInterface $loggerChannelFactory, OAuth $oauthService, StateInterface $state) {
    $this->config = $configFactory->get(HubSpotEnum::CONFIG_KEY);
    $this->logger = $loggerChannelFactory->get(HubSpotEnum::LOGGER_CHANNEL);
    $this->oauthService = $oauthService;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public function getHandler(): ?Discovery {
    $handler = $this->getHandlerWithOauth();
    if ($handler) {
      return $handler;
    }

    // Use a Private App access token if OAuth is not configured.
    return $this->getHandlerWithAccessToken();
  }

  /**
   * {@inheritdoc}
   */
  public function getHandlerWithOauth(): ?Discovery {
    $tokens = $this->state->get(HubSpotEnum::OAUTH_TOKEN_STATE_KEY);
    if (empty($tokens['access_token'])) {
      return NULL;
    }

    $accessToken = $tokens['access_token'];
    $now = new \DateTime();
    // The token expires every 6 hours. Adding 15 minutes to give plenty of
    // time for a procedure to finish.
    $now->modify('+15 minutes');
    $expires = clone($now);
    $expires->setTimestamp(($tokens['expire_date']) ?? 0);
    if ($now > $expires) {
      $accessToken = $this->oauthService->getTokensByRefresh();
    }

    return Factory::createWithAccessToken($accessToken);
  }

  /**
   * {@inheritdoc}
   */
  public function getHandlerWithAccessToken(string $token = NULL): ?Discovery {
    $token = $token ?? $this->config->get('access_key') ?? NULL;

    if (!$token) {
      return NULL;
    }

    return Factory::createWithAccessToken($token);
  }

}
