<?php

namespace Drupal\hubspot_api\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Url;
use Drupal\hubspot_api\Enum\HubSpotEnum;
use GuzzleHttp\ClientInterface;
use HubSpot\Client\Auth\OAuth\Model\TokenResponseIF;
use HubSpot\Factory;

/**
 * Provides an OAuth2 service for Hubspot.
 */
class OAuth {

  /**
   * The config factory to use.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * Logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * The state key/value store.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * Constructs a new OAuth Service.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   A Guzzle client object.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   The logger factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state.
   */
  public function __construct(ConfigFactoryInterface $configFactory, ClientInterface $httpClient, LoggerChannelFactoryInterface $loggerChannelFactory, StateInterface $state) {
    $this->configFactory = $configFactory;
    $this->httpClient = $httpClient;
    $this->logger = $loggerChannelFactory->get(HubSpotEnum::LOGGER_CHANNEL);
    $this->state = $state;
  }

  /**
   * Gets the OAuth token from the Hubspot code.
   *
   * @param string $code
   *   The code returned after authorizing the Hubspot environment for this app.
   *
   * @return \HubSpot\Client\Auth\OAuth\Model\TokenResponseIF|null
   *   The token data, or NULL otherwise.
   */
  public function getTokensByCode(string $code): ?TokenResponseIF {
    $config = $this->configFactory->get(HubSpotEnum::CONFIG_KEY);
    $clientId = $config->get('client_id');
    $clientSecret = $config->get('client_secret');

    try {
      $tokens = Factory::create()->auth()->oAuth()->tokensApi()->create(
        'authorization_code',
        $code,
        Url::fromRoute('hubspot_api.oauth_redirect', [], ['absolute' => TRUE])
          ->toString(),
        $clientId,
        $clientSecret
      );
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to get OAuth tokens: %error', ['%error' => $e->getMessage()]);
      return NULL;
    }

    return $tokens;
  }

  /**
   * Refresh the OAuth token.
   *
   * @return string|null
   *   The token data, or NULL otherwise.
   */
  public function getTokensByRefresh(): ?string {
    $config = $this->configFactory->get(HubSpotEnum::CONFIG_KEY);
    $hubspotApiTokens = $this->state->get(HubSpotEnum::OAUTH_TOKEN_STATE_KEY);
    $clientId = $config->get('client_id');
    $clientSecret = $config->get('client_secret');

    try {
      $tokens = Factory::create()->auth()->oAuth()->tokensApi()->create(
        'refresh_token',
        NULL,
        NULL,
        $clientId,
        $clientSecret,
        $hubspotApiTokens['refresh_token']
      );
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to refresh OAuth tokens: %error', ['%error' => $e->getMessage()]);
      return NULL;
    }

    $this->saveTokens($tokens);

    return $tokens->getAccessToken();
  }

  /**
   * Saves the OAuth tokens to be used for later API calls.
   *
   * @param \HubSpot\Client\Auth\OAuth\Model\TokenResponseIF $tokens
   *   The token data.
   *
   * @return void
   *   Nothing.
   *
   * @see https://developers.hubspot.com/docs/methods/oauth2/refresh-access-token
   *
   * @see https://developers.hubspot.com/docs/methods/oauth2/get-access-and-refresh-tokens
   */
  public function saveTokens(TokenResponseIF $tokens): void {
    $tokensToSaveInState = [
      'access_token' => $tokens->getAccessToken(),
      'refresh_token' => $tokens->getRefreshToken(),
      'expire_date' => $tokens->getExpiresIn() + time(),
    ];
    $this->state->set(HubSpotEnum::OAUTH_TOKEN_STATE_KEY, $tokensToSaveInState);
  }

}
